@@ -36,8 +36,8 @@ pub struct ScopedRaw;
 impl ScopedRaw {
     /// Enter raw terminal mode.
     ///
-    /// Must be left before using `log::info!(..)` or any
-    /// other printing macros or functions.
+    /// Must be left before using `log::info!(..)` or any other printing macros
+    /// or functions.
     fn new() -> Result<Self> {
         crossterm::terminal::enable_raw_mode()?;
         Ok(Self)
@@ -47,9 +47,9 @@ impl ScopedRaw {
     ///
     /// Also called on `drop`.
     pub fn restore_terminal() -> Result<()> {
-        stdout().queue(crossterm::cursor::Show)?;
         crossterm::terminal::disable_raw_mode()?;
         stdout()
+            .queue(crossterm::cursor::Show)?
             .flush()
             .wrap_err_with(|| eyre!("Failed to restore terminal"))
     }
@@ -93,6 +93,7 @@ pub(super) enum UserSelection {
 }
 
 /// Statefulness for the selection process
+#[derive(Debug)]
 struct State<'s, 't>
 where
     't: 's,
@@ -102,6 +103,8 @@ where
     /// The content the user provided for the suggestion, if any.
     pub custom_replacement: String,
     pub cursor_offset: u16,
+    /// Back ticked orignal content
+    pub backticked_original: String,
     /// Which index to show as highlighted.
     pub pick_idx: usize,
     /// Total number of pickable slots.
@@ -114,9 +117,19 @@ impl<'s, 't> From<&'s Suggestion<'t>> for State<'s, 't> {
             suggestion,
             custom_replacement: String::new(),
             cursor_offset: 0,
-            pick_idx: 0usize,
+            // TODO only suggest this if this doesn't have spaces and/or parses with `ap_syntax`
+            // TODO and check the identifiers against everything we've seen in the codebase
+            // TODO this has a few issues though, that partial runs might be unaware of all `Ident`s
+            // TODO so there should probably be a strict mode for full runs, that checks the existence
+            // TODO and the default, partial mode that is more forgiving
+            backticked_original: format!(
+                "`{}`",
+                sub_chars(suggestion.chunk.as_str(), suggestion.range.clone())
+            ),
+            // start at a suggestion, not the custom field or ticked suggestion
+            pick_idx: 1_usize + (!suggestion.replacements.is_empty()) as usize,
             // all items provided by the checkers plus the user provided
-            n_items: suggestion.replacements.len() + 1,
+            n_items: suggestion.replacements.len() + 2,
         }
     }
 }
@@ -135,27 +148,32 @@ where
         self.pick_idx = (self.pick_idx + self.n_items - 1).rem_euclid(self.n_items);
     }
 
-    /// Select the custom line, which is by definition the
-    /// last selectable.
+    /// Select the custom line, which is by definition the last selectable.
     pub fn select_custom(&mut self) {
-        self.pick_idx = self.n_items - 1;
+        self.pick_idx = 0;
     }
 
     /// Checks if the currently selected line is the custom entry.
     pub fn is_custom_entry(&self) -> bool {
-        self.pick_idx + 1 == self.n_items
+        self.pick_idx == 0
+    }
+
+    pub fn is_ticked_entry(&self) -> bool {
+        self.pick_idx == 1
     }
 
     /// Convert the replacement to a `BandAid`
     pub fn to_bandaid(&self) -> BandAid {
-        if self.is_custom_entry() {
+        if self.is_ticked_entry() {
+            BandAid::from((self.backticked_original.clone(), &self.suggestion.span))
+        } else if self.is_custom_entry() {
             BandAid::from((self.custom_replacement.clone(), &self.suggestion.span))
         } else {
             let replacement = self
                 .suggestion
                 .replacements
-                .get(self.pick_idx)
-                .expect("User Pick index is out of bounds");
+                .get(self.pick_idx.saturating_sub(2)) // there is a static offset of 2
+                .expect("User Pick index is never out of bounds. qed");
             BandAid::from((replacement.to_owned(), &self.suggestion.span))
         }
     }
@@ -171,7 +189,18 @@ pub struct UserPicked {
 impl UserPicked {
     /// Count the number of suggestions across all files in total.
     pub fn total_count(&self) -> usize {
-        self.bandaids.iter().map(|(_origin, vec)| vec.len()).sum()
+        self.bandaids
+            .iter()
+            .map(|(_origin, bandaids)| bandaids.len())
+            .sum()
+    }
+
+    /// Are there any user picks?
+    pub fn is_empty(&self) -> bool {
+        self.bandaids
+            .iter()
+            .find(|(_origin, bandaids)| !bandaids.is_empty())
+            .is_none()
     }
 
     /// Apply a single `BandAid`
@@ -183,8 +212,7 @@ impl UserPicked {
     }
 
     /// Apply multiple bandaids.
-    #[allow(unused)]
-    fn add_bandaids<I>(&mut self, origin: &ContentOrigin, fixes: I)
+    pub fn add_bandaids<I>(&mut self, origin: &ContentOrigin, fixes: I)
     where
         I: IntoIterator<Item = BandAid>,
     {
@@ -195,6 +223,16 @@ impl UserPicked {
             .extend(iter);
     }
 
+    /// Join two `UserPick`s.
+    pub fn extend(&mut self, other: Self) {
+        self.bandaids.extend(other.bandaids.into_iter());
+    }
+
+    #[allow(dead_code)]
+    fn wrap_in_ticks(&self, _state: &mut State, _event: KeyEvent) -> Result<UserSelection> {
+        unimplemented!()
+    }
+
     /// Provide a replacement that was not provided by the backend
     fn enter_custom_replacement(
         &self,
@@ -249,7 +287,7 @@ impl UserPicked {
     // .. suggestion0 [suggestion1] suggestion2 suggestion3 ..
     // arrow left
     // .. suggestion1 [suggestion2] suggestion3 suggestion4 ..
-    // but now it's only a very simple list for now
+    // but now it's only a very simple list
     fn print_replacements_list(&self, state: &mut State) -> Result<()> {
         let mut stdout = stdout();
 
@@ -272,8 +310,8 @@ impl UserPicked {
 
         // render all replacements in a vertical list
 
-        stdout.queue(cursor::SavePosition).unwrap();
-        let _ = stdout.flush();
+        stdout.queue(cursor::SavePosition)?;
+        let _ = stdout.queue(cursor::MoveDown(1))?;
 
         let active_idx = state.pick_idx;
 
@@ -282,89 +320,60 @@ impl UserPicked {
         } else {
             state.custom_replacement.as_str()
         };
-        if state.n_items != active_idx + 1 {
-            stdout
-                .queue(cursor::MoveUp(1))
-                .unwrap()
-                .queue(terminal::Clear(terminal::ClearType::CurrentLine))
-                .unwrap()
-                .queue(cursor::MoveToColumn(4))
-                .unwrap()
-                .queue(PrintStyledContent(StyledContent::new(
-                    custom,
-                    custom_content,
-                )))
-                .unwrap();
-        } else {
-            stdout
-                .queue(cursor::MoveUp(1))
-                .unwrap()
-                .queue(terminal::Clear(terminal::ClearType::CurrentLine))
-                .unwrap()
-                .queue(cursor::MoveToColumn(2))
-                .unwrap()
-                .queue(PrintStyledContent(StyledContent::new(tick.clone(), '»')))
-                .unwrap()
-                .queue(cursor::MoveToColumn(4))
-                .unwrap()
-                .queue(PrintStyledContent(StyledContent::new(
-                    custom,
-                    custom_content,
-                )))
-                .unwrap();
-        }
-        let _ = stdout.flush();
 
-        state
-            .suggestion
-            .replacements
-            .iter()
+        std::iter::once((&custom, custom_content))
+            .chain(std::iter::once((
+                &others,
+                state.backticked_original.as_str(),
+            )))
+            .chain(
+                state
+                    .suggestion
+                    .replacements
+                    .iter()
+                    .map(|s| (&others, s.as_str())),
+            )
             .enumerate()
-            .for_each(|(idx, replacement)| {
-                let idx = idx as u16;
-                if idx != active_idx as u16 {
-                    // TODO figure out a way to deal with those errors better
-                    stdout
-                        .queue(cursor::MoveUp(1))
-                        .unwrap()
-                        .queue(terminal::Clear(terminal::ClearType::CurrentLine))
-                        .unwrap()
-                        .queue(cursor::MoveToColumn(4))
-                        .unwrap()
-                        .queue(PrintStyledContent(StyledContent::new(
-                            others.clone(),
-                            replacement.clone(),
-                        )))
-                        .unwrap();
+            .map(|(idx, (style, content))| {
+                (
+                    idx,
+                    PrintStyledContent(StyledContent::new(style.clone(), content.clone())),
+                )
+            })
+            .try_fold(&mut stdout, |cmd, (idx, mut item)| {
+                let cmd = cmd
+                    .queue(cursor::MoveUp(1))?
+                    .queue(terminal::Clear(terminal::ClearType::CurrentLine))?;
+
+                if idx == active_idx {
+                    *item.0.style_mut() = highlight;
+                    if idx == 0 {
+                        cmd.queue(crossterm::cursor::Show)?;
+                    } else {
+                        cmd.queue(crossterm::cursor::Hide)?;
+                    }
+                    cmd.queue(cursor::MoveToColumn(2))?
+                        .queue(PrintStyledContent(StyledContent::new(tick, '»')))?
+                        .queue(cursor::MoveToColumn(4))?
                 } else {
-                    stdout
-                        .queue(cursor::MoveUp(1))
-                        .unwrap()
-                        .queue(terminal::Clear(terminal::ClearType::CurrentLine))
-                        .unwrap()
-                        .queue(cursor::MoveToColumn(2))
-                        .unwrap()
-                        .queue(PrintStyledContent(StyledContent::new(tick.clone(), '»')))
-                        .unwrap()
-                        .queue(cursor::MoveToColumn(4))
-                        .unwrap()
-                        .queue(PrintStyledContent(StyledContent::new(
-                            highlight.clone(),
-                            replacement.clone(),
-                        )))
-                        .unwrap();
+                    cmd.queue(cursor::MoveToColumn(4))?
                 }
-            });
+                .queue(item)
+            })?;
 
-        stdout.queue(cursor::RestorePosition).unwrap();
+        stdout.queue(cursor::RestorePosition)?.flush()?;
 
-        let _ = stdout.flush();
         Ok(())
     }
 
     /// Wait for user input and process it into a `UserSelection` enum.
-    fn user_input(&self, state: &mut State, running_idx: (usize, usize)) -> Result<UserSelection> {
-        {
+    fn user_input(
+        &self,
+        state: &mut State,
+        running_idx: usize,
+        total: usize,
+    ) -> Result<UserSelection> {
+        let skip = {
             let _guard = ScopedRaw::new();
 
             let mut boring = ContentStyle::new();
@@ -373,8 +382,8 @@ impl UserPicked {
 
             let question = format!(
                 "({nth}/{of_n}) Apply this suggestion [y,n,q,a,d,j,e,?]?",
-                nth = running_idx.0 + 1,
-                of_n = running_idx.1
+                nth = running_idx + 1,
+                of_n = total
             );
 
             // a new suggestion, so prepare for the number of items that are visible
@@ -385,26 +394,20 @@ impl UserPicked {
             // erase this many lines of the regular print
             const ERASE: u16 = 4;
             // lines used by the question
-            const QUESTION: u16 = 4;
-            let extra_rows_to_flush = (state.n_items - (ERASE - QUESTION) as usize) as u16;
+            const QUESTION: u16 = 3;
+            let extra_rows_to_flush =
+                (state.n_items.saturating_sub((ERASE - QUESTION) as usize)) as u16;
             stdout()
-                .queue(cursor::Hide)
-                .unwrap()
-                .queue(cursor::MoveUp(ERASE)) // erase the 5 last lines of suggestion print
-                .unwrap()
-                .queue(terminal::Clear(terminal::ClearType::FromCursorDown))
-                .unwrap()
-                .queue(cursor::MoveDown(1)) // add a space between the question and the error
-                .unwrap()
-                .queue(PrintStyledContent(StyledContent::new(boring, question)))
-                .unwrap()
-                .queue(terminal::ScrollUp(extra_rows_to_flush))
-                .unwrap()
-                .queue(cursor::MoveToColumn(0))
-                .unwrap()
-                .queue(cursor::MoveDown(extra_rows_to_flush))
-                .unwrap();
-        }
+                .queue(cursor::Hide)?
+                .queue(cursor::MoveUp(ERASE))? // erase the 5 last lines of suggestion print
+                .queue(terminal::Clear(terminal::ClearType::FromCursorDown))?
+                .queue(cursor::MoveDown(1))? // add a space between the question and the error
+                .queue(PrintStyledContent(StyledContent::new(boring, question)))?
+                .queue(terminal::ScrollUp(extra_rows_to_flush))?
+                .queue(cursor::MoveToColumn(0))?
+                .queue(cursor::MoveDown(extra_rows_to_flush))?;
+            ERASE - QUESTION
+        };
 
         loop {
             let mut _guard = ScopedRaw::new();
@@ -412,17 +415,12 @@ impl UserPicked {
             self.print_replacements_list(state)?;
 
             if state.is_custom_entry() {
-                info!("Custom entry mode");
-
-                stdout().queue(cursor::SavePosition).unwrap();
+                stdout().queue(cursor::SavePosition)?;
                 stdout()
-                    .queue(cursor::Show)
-                    .unwrap()
-                    .queue(cursor::MoveToPreviousLine(1))
-                    .unwrap()
-                    .queue(cursor::MoveToColumn(4 + state.cursor_offset))
-                    .unwrap();
-                let _ = stdout().flush();
+                    .queue(cursor::Show)?
+                    .queue(cursor::MoveToPreviousLine(1 - skip))?
+                    .queue(cursor::MoveToColumn(4 + state.cursor_offset))?;
+                stdout().flush()?;
             }
 
             let event = match crossterm::event::read()
@@ -447,10 +445,8 @@ impl UserPicked {
                 let pick = self.enter_custom_replacement(state, event)?;
 
                 stdout()
-                    .queue(cursor::Hide)
-                    .unwrap()
-                    .queue(cursor::RestorePosition)
-                    .unwrap();
+                    .queue(cursor::Hide)?
+                    .queue(cursor::RestorePosition)?;
 
                 match pick {
                     UserSelection::Nop => continue,
@@ -493,74 +489,69 @@ impl UserPicked {
     }
 
     pub(super) fn select_interactive<'s>(
-        suggestions_per_path: SuggestionSet<'s>,
-        _config: &Config,
+        origin: ContentOrigin,
+        suggestions: Vec<Suggestion<'s>>,
     ) -> Result<(Self, UserSelection)> {
+        let count = suggestions.len();
         let mut picked = UserPicked::default();
 
-        trace!("Select the ones to actully use");
+        let mut suggestions_it = suggestions.iter().enumerate();
+        let start = suggestions_it.clone();
 
-        for (origin, suggestions) in suggestions_per_path {
-            let count = suggestions.len();
-            trace!("Path is {} and has {}", origin, count);
+        // TODO make use of it
+        let direction = Direction::Forward;
+        'outer: loop {
+            let opt_next = match direction {
+                Direction::Forward => suggestions_it.next(),
+                // FIXME TODO this is just plain wrong
+                Direction::Backward => suggestions_it.next_back(),
+            };
 
-            // TODO juck, uggly
-            let mut suggestions_it = suggestions.clone().into_iter().enumerate();
+            trace!("next() ---> {:?}", &opt_next);
 
-            let mut direction = Direction::Forward;
-            loop {
-                let opt: Option<(usize, Suggestion)> = match direction {
-                    Direction::Forward => suggestions_it.next(),
-                    Direction::Backward => suggestions_it.next_back(), // FIXME TODO this is just plain wrong
-                };
-
-                trace!("next() ---> {:?}", &opt);
-
-                if opt.is_none() {
-                    match direction {
-                        Direction::Forward => {
-                            trace!("completed file, continue to next");
-                            break; // we completed this file, move on to the next
-                        }
-                        Direction::Backward => {
-                            trace!("went back, now back at the beginning");
-                            suggestions_it = suggestions.clone().into_iter().enumerate();
-                            continue;
-                        } // go to the start
+            let (idx, suggestion) = match opt_next {
+                Some(x) => x,
+                None => match direction {
+                    Direction::Forward => {
+                        trace!("completed file, continue to next");
+                        break; // we completed this file, move on to the next
                     }
-                }
-                let (idx, suggestion) = opt.expect("Must be Some(_)");
-                if suggestion.replacements.is_empty() {
-                    trace!("Suggestion did not contain a replacement, skip");
-                    continue;
-                }
-                println!("{}", suggestion);
+                    Direction::Backward => {
+                        trace!("went back, now back at the beginning");
+                        suggestions_it = start.clone();
+                        continue;
+                    } // go to the start
+                },
+            };
+            if suggestion.replacements.is_empty() {
+                trace!("BUG: Suggestion did not contain a replacement, skip");
+                continue;
+            }
+            println!("{}", suggestion);
 
-                let mut state = State::from(&suggestion);
+            let mut state = State::from(suggestion);
 
-                let mut pick = picked.user_input(&mut state, (idx, count))?;
-                while pick == UserSelection::Help {
-                    println!("{}", HELP);
-                    pick = picked.user_input(&mut state, (idx, count))?;
-                }
-                match pick {
+            'inner: loop {
+                match picked.user_input(&mut state, idx, count)? {
                     usel @ UserSelection::Abort | usel @ UserSelection::Quit => {
-                        return Ok((picked, usel))
+                        let _ = ScopedRaw::restore_terminal();
+                        return Ok((picked, usel));
                     }
-                    UserSelection::SkipFile => break, // break the inner loop
+                    UserSelection::SkipFile => break 'outer,
                     UserSelection::Previous => {
-                        unimplemented!("Requires a iterator which works bidrectionally")
+                        warn!("Requires a iterator which works bidrectionally");
+                        continue 'inner;
                     }
                     UserSelection::Help => {
-                        unreachable!("Help must not be reachable here, it is handled before")
+                        println!("{}", HELP);
+                        continue 'inner;
                     }
                     UserSelection::Replacement(bandaid) => {
                         picked.add_bandaid(&origin, bandaid);
                     }
-                    _ => continue,
+                    UserSelection::Nop | UserSelection::Skip => {}
                 };
-
-                direction = Direction::Forward;
+                break 'inner;
             }
         }
         Ok((picked, UserSelection::Nop))
